/*
		I'm not a big fan of making maps for small testing purposes, so I'll just set
	world.maxx and world.maxy and let DreamMaker handle the rest of the details for me,
	based on the definitions I give it of atoms and, more specifically, turfs.

		For more information of any of the world variables, click your cursor on the
	variable name and press F1.
*/

world
	name = "Linked Objects Demo by Hiead"
	maxx = 13
	maxy = 13

/*
		For simplicity of the demo, I'm only using one icon with one icon state: a black
	square. Objects that will vary in color will define their custom colorings in the
	New() methods, respectively. For more information on New(), icon or, more generally,
	basically anything that you see in this demo, press F1 while your cursor is on the
	item you would like help with, and read up!

		Nearly all types derive from /atom. I will take advantage of this by assigning all
	of those types with an icon of icon.dmi. The /atom/movable type is the parent of all
	objects that can move around. For this type, let us define one variable: a child object.

		The child object (ChildObject) will be of the /atom/movable type, and will belong
	to types of /atom/movable. When a descendant of /atom/movable moves (via Move()), its
	child object moves. This will incur a chain reaction such that if this child object has
	a child object of its own, the child object will move. This will happen until the entire
	series is finished with movement. It should be noted that further measures might be taken
	to be assured that the child object of one object does not have a child object
	as a reference to its parent object: an infinite loop! As an example, if object A
	referenced object B as a child object, and B referenced A, then when A moved, B would move,
	causing A to move, and so on. Infinite loops such as this are quick ways to crash the
	program.
*/

atom
	icon = 'icon.dmi'

	movable
		animate_movement = 0	// We're just using squares; it's prettier to not animate!

		var/atom/movable/ChildObject = null

/*
		In Add_Link(), we will first want to determine the end of the chain, so as to
	prevent adding a link to the wrong part of the chain. To do this, we will work our way
	down the chain until we reach a link that does not have a child object.

		After finding the final	link, we will create a new link in that link's location,
	and assign the then-final link's child object to the newly created link.
*/
		proc/Add_Link()
			var/atom/movable/testLink = src		// src is the start of the chain test
			while(testLink.ChildObject)			// Test for finality of chain
				testLink = testLink.ChildObject

			var/obj/Link/newLink = new(testLink.loc)	// Create new link
			testLink.ChildObject = newLink

		Move()	// Called when a descendant of /atom/movable moves via Move().
			var/last_loc = loc	// last_loc is the location prior to movement.

			. = ..()
			//^ Sets the return variable (.) to the return value of the default action

			if(ChildObject && .)	// If this object has a child, and movement succeeded...
				ChildObject.Move(last_loc)	// Move the child to the last location

turf/New()
	..()
	icon += "#00ff00"	// Turfs will be green

/*
		For the individual links of the chain, we will use a user-defined type of /obj/Link.
	When this object is created, it will color itself totally randomly, except for the
	green parameter, which is limited to 128 to prevent confusion with the turf's green.
*/

obj/Link
	New()
		..()
		icon += rgb(rand(0,255), rand(0,128), rand(0,255))

client/verb/Additional_Link()	// Allow players to add new links to their own mob's chain.
	mob.Add_Link()